# OME/Web/DBObjDetail/__OME_ModuleExecution.pm

#-------------------------------------------------------------------------------
#
# Copyright (C) 2003 Open Microscopy Environment
#       Massachusetts Institute of Technology,
#       National Institutes of Health,
#       University of Dundee
#
#
#
#    This library is free software; you can redistribute it and/or
#    modify it under the terms of the GNU Lesser General Public
#    License as published by the Free Software Foundation; either
#    version 2.1 of the License, or (at your option) any later version.
#
#    This library is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#    Lesser General Public License for more details.
#
#    You should have received a copy of the GNU Lesser General Public
#    License along with this library; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#-------------------------------------------------------------------------------




#-------------------------------------------------------------------------------
#
# Written by:    Josiah Johnston <siah@nih.gov>
#
#-------------------------------------------------------------------------------


package OME::Web::DBObjDetail::__OME_ModuleExecution;

=pod

=head1 NAME

OME::Web::DBObjDetail::__OME_ModuleExecution - Show details of a ModuleExecution

=cut

#*********
#********* INCLUDES
#*********

use strict;
use OME;
our $VERSION = $OME::VERSION;
use Log::Agent;
use HTML::Template;

use OME::Web::DBObjTable;
use OME::Tasks::ModuleExecutionManager;

#*********
#********* GLOBALS AND DEFINES
#*********

$VERSION = $OME::VERSION;
use base qw(OME::Web::DBObjDetail);

#*********
#********* PUBLIC METHODS
#*********

sub getPageBody {
	my $self = shift;
	my $session = $self->Session();
	my $factory = $session->Factory();
	my $q = $self->CGI();

	my $html = ( $self->_takeAction( ) || '' );

	my $mex = $self->_loadObject();
	return $self->SUPER::getPageBody()
		if $mex->virtual_mex();

	$self->{ form_name } = 'primary';
	$html .= $q->startform( -name => $self->{ form_name }  ).
	         $q->hidden(-name => 'Type', -default => $q->param( 'Type' ) ).
	         $q->hidden(-name => 'ID', -default => $q->param( 'ID' ) );
	
	my @actual_inputs = $mex->inputs();
	my @formal_outputs = $mex->module()->outputs();
	@formal_outputs = grep( defined $_->semantic_type(), @formal_outputs );
	my @formal_inputs = $mex->module()->inputs();
	my @untyped_outputs = $mex->untypedOutputs();
	my $tmpl = OME::Web::TemplateManager->getRenderingTemplate(
	    ref($mex),'detail1','one');
	my %tmpl_data;
	my $tableMaker = OME::Web::DBObjTable->new( CGI => $q );	
	
	############
	#	 Merge outputs
	
	# pick up the previously stored merged groups of outputs
	my @merged_output_groups;
	if( $q->param( 'merged_output_groups' ) ) {
		@merged_output_groups = $q->param( 'merged_output_groups' );
		# remove groups as requested
		@merged_output_groups = grep( $_ ne $q->param( 'UngroupID' ), @merged_output_groups );
		# parse the comma delimited ids
		@merged_output_groups = map( [ split( ',', $_ ) ], @merged_output_groups );
		# turn ids into objects for each merge group
		foreach my $merged_output( @merged_output_groups ) {
			$merged_output = [ map( 
				$factory->loadObject( "OME::Module::FormalOutput", $_ ), 
				@$merged_output ) ];
		}
	}
	
	# pick up recent merge request & store it
	if( $q->param( 'Merge Outputs' ) && $q->param( 'formal_outputs_to_merge' )) {
		my @formal_outputs_to_merge = $q->param( 'formal_outputs_to_merge' );
		# load the formal output object for each id
		@formal_outputs_to_merge = map( 
			$factory->loadObject( "OME::Module::FormalOutput", $_ ), 
			@formal_outputs_to_merge );
		# add to the list of groups
		push( @merged_output_groups, \@formal_outputs_to_merge );
	}
	
	# save formal output groupings 
	my @merged_output_groups_id;
	# make an array that looks like ('id,id,id', 'id,id')
	foreach my $output_set ( @merged_output_groups ) {
		push @merged_output_groups_id, join( ',', map( $_->id(), @$output_set ) );
	}
 	$html .= $q->hidden(
		-name => 'merged_output_groups', 
		-default => \@merged_output_groups_id, 
		-override => 1 
	);


	##############
	#	Start Rendering


	# determine what permissions (if any) the user can edit,
	# and compile html snippet to allow editing. These snippets
	# are inserted in the MEX detail template as run time parameters.
	my $chown_options;
	my $exp           = $session->experimenter();
	my $super_user_id = $session->Configuration()->super_user;
	# experimenter: only the superuser can change the experimenter ownership
	if( $exp->id eq $super_user_id ) {
		my @experimenters = $factory->findObjects( '@Experimenter' );
		$chown_options->{ 'chown_experimenter' } = 
			$self->Renderer()->renderArray( \@experimenters, 'dropdown_select', 
				{ field_name => 'experimenter', default_value => $mex->experimenter_id, type => '@Experimenter' } );
	}
	# group: the superuser or owner can change the group ownership
	if( ( $exp->id eq $mex->experimenter_id ) or ($exp->id eq $super_user_id ) ) {
		my @groups = $factory->findObjects( '@Group' );
		$chown_options->{ 'chown_group' } = 
			$self->Renderer()->renderArray( \@groups, 'dropdown_select', 
				{ field_name => 'group', default_value => $mex->group_id, type => '@Group' } );
	}
	# Render ;Execution Details
	$tmpl_data{ mex_detail } = $self->Renderer()->render( $mex, 'detail', $chown_options);



	# I/O Table
	# input column of i/o table
	$tmpl_data{ input_names } = [ map( { name => $q->a( { href => '#'.$_->name() }, $_->name() ) }, @formal_inputs) ];
	# output column of i/o table
	$tmpl_data{ output_names } = [ map( { 
		name => $q->a( { href => '#'.$_->name() }, $_->name() ),
		checkbox => $q->checkbox( -name => 'formal_outputs_to_merge', -value => $_->id(), -label => '', -checked => undef )
	}, @formal_outputs) ];
	# untyped outputs column of i/o table
	$tmpl_data{ untyped_output_names } = [ map( { 
		name => $q->td( $q->a( { href => '#untyped_'.$_->semantic_type->name() }, $_->semantic_type->name() ) ),
#		checkbox => $q->checkbox( -name => 'untyped_outputs_to_merge', -value => $_->id(), -label => '', -checked => undef )
	}, @untyped_outputs) ];
	# Ouput Groups column
	foreach my $group ( @merged_output_groups ) {
		push( @{ $tmpl_data{ output_groups } }, { 
		group => $q->a( { href => '#'.join( ',', map( $_->id(), @$group ) ) }, join( ', ', map( $_->name(), @$group ) ) ),
		# Should I add an ungroup button here?
		} );
	}
	# button to merge outputs
	$tmpl_data{ merge_button } = $q->submit( -name => 'Merge Outputs' );
	
	# Merged outputs.
	foreach my $group( @merged_output_groups ) {
		my @table_data;
		my $group_id = join( ',', map( $_->id, @{$group} ) );
		foreach my $fo ( @{$group} ) {
			next unless $fo->semantic_type;
			push @table_data, {
				title   => $fo->name(),
				type    => $self->_STformalName( $fo->semantic_type() ),
				search  => {module_execution_id => $mex->id}
			};
		}
		my @tables = $tableMaker->getJoinTable( 
			{
				excludeFields    => { module_execution => undef },
				embedded_in_form => $self->{ form_name },
				group_id         => $group_id
			},
			\@table_data
		);
		push( @{ $tmpl_data{ grouped_outputs } }, { output => 
			$q->a( { 
				-title => 'Click here to Ungroup these Outputs', 
				-href  => "javascript: document.forms['".$self->{ form_name }."'].elements['UngroupID'].value='$group_id'; document.forms['".$self->{ form_name }."'].submit();", 
			}, "Ungroup Outputs" )."<br>".
			$q->a( { -name => $group_id }, ' ').
			join( '<br>', @tables ) 
		} );
	}
	
	# Render outputs ( ungrouped )
	foreach my $fo ( @formal_outputs ) {
		my $more_info_url = $self->getSearchURL( 
			$fo->semantic_type()->requireAttributeTypePackage()->getFormalName(),
			module_execution => $mex->id()
		);

		push( @{ $tmpl_data{ outputs } }, { 
			output => $tableMaker->getTable( 
				{
					excludeFields    => { module_execution => undef },
					noSearch => 1,
					title    => $fo->name()." <a href='$more_info_url'>Search within these outputs</a>",
					embedded_in_form => $self->{ form_name },
					table_id => "Formal_Output_".$fo->id
				},
				$self->_STformalName( $fo->semantic_type() ),
				{ module_execution => $mex }
			)
		} );
		$tmpl_data{ popup_overlay } = $mex->id() if $fo->name() eq 'Location';
	}
		
	# Untyped Outputs Tables
	foreach my $sto ( @untyped_outputs ) {
		my $attributes = OME::Tasks::ModuleExecutionManager->
			getAttributesForMEX($mex,$sto->semantic_type);
		push( @{ $tmpl_data{ untyped_outputs } }, { untyped_output => 
			$q->a( { -name => 'untyped_'.$sto->semantic_type->name() }, ' ').
			$tableMaker->getTable( 
				{
					excludeFields    => { module_execution => undef },
					embedded_in_form => $self->{ form_name },
					title            => $sto->semantic_type->name(),
				},
				$self->_STformalName( $sto->semantic_type() ),
				$attributes
			)
		} );
	}

	# Actual Inputs Tables
	foreach my $ai ( @actual_inputs ) {
		next unless $ai->input_module_execution();
		my $attributes = OME::Tasks::ModuleExecutionManager->
			getAttributesForMEX($ai->input_module_execution,$ai->formal_input()->semantic_type);
		push( @{ $tmpl_data{ inputs } }, { input => 
			$q->a( { -name => $ai->formal_input()->name() }, ' ' ).
			$tableMaker->getTable( 
				{
					excludeFields    => { module_execution => undef },
					embedded_in_form => $self->{ form_name },
					title            => "The <span class='ome_punchline'>".$ai->formal_input()->name()."</span> input was satisfied by ".
					                    $self->Renderer()->render( $ai->input_module_execution, 'ref' ),
				},
				$self->_STformalName( $ai->formal_input()->semantic_type() ),
				$attributes
			)
		} );
	}
	
	# populate the template & print it out
	$tmpl->param( %tmpl_data );
	$q->delete( 'UngroupID' );
	$html .= $tmpl->output().
	         $q->hidden(-name => 'UngroupID' ).
	         # For the DBObjTable column ordering
	         $q->hidden(-name => 'action' ).
	         $q->endform();

	$self->_takeAction( );

	return ('HTML', $html);
}

sub _STformalName {
	my ($self, $ST) = @_;
	return '@'.$ST->name();
}

=head2 _takeAction

Allows an administrator to change the experimenter and group ownership.
Allows an owner of a MEX to change group ownership.

=cut

sub _takeAction {
	my $self    = shift;
	my $session = $self->Session();
	my $factory = $session->Factory();
	my $q       = $self->CGI();
	my $mex     = $self->_loadObject();
	my $exp     = $session->experimenter();
	my $super_user_id = $session->Configuration()->super_user;

	# change group ownership
	return unless( ( $exp->id eq $mex->experimenter_id ) or ($exp->id eq $super_user_id ) );
	my $new_group = $q->param( 'group' );
	if( ( $new_group ) && ( $new_group ne $mex->group_id ) ) {
		$mex->group( $new_group );
		$mex->storeObject();
		$session->commitTransaction();
	}

	# change experimenter ownership
	return unless( $exp->id eq $super_user_id );
	my $new_exp   = $q->param( 'experimenter' );
	if( ( $new_exp ) && ( $new_exp ne $mex->experimenter_id ) ) {
		$mex->experimenter( $new_exp );
		$mex->storeObject();
		$session->commitTransaction();
	}


}

=head1 Author

Josiah Johnston <siah@nih.gov>

=cut

1;
